@@ -4,6 +4,7 @@ from django.views.generic.edit import (FormMixin, ProcessFormView,
 from django_modalview.generic.base import (ModalContextMixin, ModalView,
                                            ModalTemplateMixin, ModalUtilMixin)
 from django_modalview.generic.component import (FORM_TEMPLATE_CONTENT,
+                                                LAST_FORM_TEMPLATE,
                                                 FORM_TEMPLATE, ModalButton)
 
 
@@ -20,11 +21,13 @@ class ModalEditContextMixin(ModalContextMixin):
         self.action = None
         self.content_template_name = FORM_TEMPLATE_CONTENT
         self.submit_button = ModalButton(value='send', button_type='primary')
+        self.form_content_template_name = LAST_FORM_TEMPLATE
 
     def get_context_modal_data(self, **kwargs):
         kwargs.update({
             'submit_button': self.submit_button,
             'action': self.action,
+            'form_template_name': self.form_content_template_name
         })
         return super(ModalEditContextMixin,
                      self).get_context_modal_data(**kwargs)
@@ -45,10 +48,7 @@ class ModalFormMixin(ModalEditContextMixin, FormMixin):
 
     def _form_response(self, **kwargs):
         kwargs.update(self.get_context_modal_data())
-        return self.render_to_response(
-            is_ajax=self.is_ajax,
-            context=kwargs
-        )
+        return self.render_to_response(context=kwargs)
 
     def form_valid(self, form, **kwargs):
         self._can_redirect = True
@@ -58,6 +58,34 @@ class ModalFormMixin(ModalEditContextMixin, FormMixin):
         return self._form_response(**kwargs)
 
 
+class ModalFormUtilMixin(ModalFormMixin, ModalUtilMixin):
+    """
+        Mixin that provide a way to show and to handle a modal with a django
+        form
+    """
+    def util_on_form_valid(self, **kwargs):
+        pass
+
+    def util_on_form_invalid(self, **kwargs):
+        pass
+
+    def dispatch(self, request, *args, **kwargs):
+        self.kwargs.update(kwargs)
+        self.kwargs.update(request.GET)
+        return super(ModalFormUtilMixin, self).dispatch(request, *args,
+                                                        **kwargs)
+
+    def form_valid(self, form, **kwargs):
+        self.kwargs.update(**form.cleaned_data)
+        self.get_util('util_on_form_valid', **self.kwargs)
+        return super(ModalFormUtilMixin, self).form_valid(form, **kwargs)
+
+    def form_invalid(self, form, **kwargs):
+        self.kwargs.update(**form.cleaned_data)
+        self.get_util('util_on_form_invalid', **self.kwargs)
+        return super(ModalFormUtilMixin, self).form_invalid(form, **kwargs)
+
+
 class ModalModelFormMixin(ModalFormMixin, ModelFormMixin):
 
     """
@@ -117,10 +145,7 @@ class ProcessModalPostView(BaseProcessModalView):
         self.template_name = self.content_template_name
         self._can_redirect = True
         kwargs.update(self.get_context_modal_data())
-        return self.render_to_response(
-            is_ajax=self.is_ajax,
-            context=kwargs
-        )
+        return self.render_to_response(context=kwargs)
 
 
 class ModalDeletionMixin(ModalEditContextMixin):
@@ -133,6 +158,29 @@ class ModalDeletionMixin(ModalEditContextMixin):
         self.object.delete()
 
 
+class ModalPostMixin(ModalEditContextMixin):
+    '''
+        A mixin that provide a way to handle a post request
+    '''
+
+
+class ModalPostUtilMixin(ModalPostMixin, ModalUtilMixin):
+
+    def util_on_post(self, *args, **kwargs):
+        pass
+
+    def dispatch(self, request, *args, **kwargs):
+        self.kwargs.update(kwargs)
+        self.kwargs.update(request.GET)
+        return super(ModalPostUtilMixin, self).dispatch(request, *args,
+                                                        **kwargs)
+
+    def post(self, request, *args, **kwargs):
+        self.kwargs.update(kwargs)
+        self.get_util('util_on_post', **self.kwargs)
+        return super(ModalPostUtilMixin, self).post(request, **kwargs)
+
+
 class BaseModalUpdateView(ModalModelFormMixin, ProcessModalFormView):
 
     """
@@ -176,7 +224,7 @@ class BaseModalDeleteView(ModalDeletionMixin, ProcessModalPostView):
         return super(BaseModalDeleteView, self).post(request, *args, **kwargs)
 
 
-class BaseModalPostView(ProcessModalPostView):
+class BaseModalPostView(ModalPostMixin, ProcessModalPostView):
 
     """
             A base view that provide a way to handle a modal that can send a post
@@ -184,6 +232,13 @@ class BaseModalPostView(ProcessModalPostView):
     """
 
 
+class BaseModalPostUtilView(ModalPostUtilMixin, ProcessModalPostView):
+    """
+            A base view that provide a way to handle a modal that can send a post
+            request, run an util and handle its response.
+    """
+
+
 class BaseModalFormView(ModalFormMixin, ProcessModalFormView):
 
     """
@@ -191,6 +246,13 @@ class BaseModalFormView(ModalFormMixin, ProcessModalFormView):
     """
 
 
+class BaseModalFormUtilView(ModalFormUtilMixin, ProcessModalFormView):
+
+    """
+        A base view that provide a way to handle a modal with form and util
+    """
+
+
 class ModalFormView(ModalTemplateMixin, BaseModalFormView):
 
     """
@@ -198,6 +260,13 @@ class ModalFormView(ModalTemplateMixin, BaseModalFormView):
     """
 
 
+class ModalFormUtilView(ModalTemplateMixin, BaseModalFormUtilView):
+
+    """
+        A view to handle a modal with form/util and to render it.
+    """
+
+
 class ModalCreateView(ModalTemplateMixin, BaseModalCreateView):
 
     """
@@ -212,14 +281,18 @@ class ModalUpdateView(ModalTemplateMixin, BaseModalUpdateView):
     """
 
 
-class ModalPostView(ModalEditContextMixin, ModalTemplateMixin,
-                    BaseModalPostView):
+class ModalPostView(ModalTemplateMixin, BaseModalPostView):
 
     """
             A view to handle a modal with a post request sender and to render it.
     """
 
 
+class ModalPostUtilView(ModalTemplateMixin, BaseModalPostUtilView):
+    """
+    """
+
+
 class ModalDeleteView(ModalTemplateMixin, BaseModalDeleteView):
 
     """
